#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent), ui(new Ui::MainWindow)
{
  ui->setupUi(this);

  tthread = new TaperThread;
  tthread->ambienttemp=295;
  tthread->current_job=tthread->requested_job=0;

  plot = new RenderArea(ui->graph);
  ui->gridLayout->addWidget(plot,1,1,0);

  plot->fiber=tthread->fiber;

  splashtimer=5;

  params_updated();
  tthread->start();
  QTimer::singleShot(1000, this, SLOT(anim()));
}

MainWindow::~MainWindow()
{
  delete ui;
}







void MainWindow::params_updated() //update the parameters in 'taperthread' and 'renderarea'
{
  //update tthread params
  tthread->fiber_segments = ui->segments->value();
  tthread->taper_nofpulls = ui->steps->value();

  tthread->fiber_r0 = ui->r0->value();
  tthread->taper_x0 = ui->x0->value()*1000.0;
  tthread->taper_L0 = ui->L0->value()*1000.0;
  tthread->taper_L1 = ui->L1->value();
  tthread->taper_L2 = ui->L2->value()*1.0E-4;
  tthread->taper_L3 = ui->L3->value()*1.0E-8;
  tthread->taper_L4 = ui->L4->value()*1.0E-12;
  tthread->taper_L5 = ui->L5->value()*1.0E-16;

  tthread->laser_power = ui->laserpower->value();
  tthread->laser_w0 = ui->w0->value();

  tthread->fiber_absorption = ui->absorption->value();
  tthread->fiber_epsilon = ui->blackness->value();

  tthread->viscosity_A  = ui->A->value();
  tthread->viscosity_B  = ui->B->value();
  tthread->viscosity_T0 = ui->T0->value();

  //tthread->updated=2;//set to 2 if fiber-array changed to indicate that plotting is requred.



  //update plot params
  ui->result_angle->setText("N/A");
  ui->result_maxtemp->setText("N/A");
  ui->result_diameter->setText("N/A");
  ui->result_length->setText("N/A");

  plot->segments=tthread->fiber_segments;



  tthread->requested_job++; //this will initiate a new tapering run.
  plot->segments=tthread->fiber_segments;
  if(tthread->isFinished())tthread->start();
  plot->update();
}




void MainWindow::on_actionAbout_Tapersim_triggered()
{
  if(about_splash!=NULL)
  {
    about_splash = new QSplashScreen;
    about_splash->setPixmap(QPixmap(":/images/splash.png"));
    //about_splash->setWindowModality(Qt::ApplicationModal);
    about_splash->show();
  //  connect(about_splash,SIGNAL(destroyed()),this,SLOT(splash_destroyed()));
  }
}
void MainWindow::splash_destroyed()
{
  about_splash->deleteLater();
}

void MainWindow::on_actionQuit_triggered()
{
  exit(0);
}


void MainWindow::redraw_slot()
{
  plot->update();
}

void MainWindow::anim()
{

  if(tthread->updated!=0)
  {
    QVariant v;
    v.setValue((int)tthread->max_temp-273);
    ui->result_maxtemp->setText(v.toString()+" [C]");

    v.setValue(0.05*((int unsigned)(10.0*plot->fiber_mindiam))); //0.05: radius; 0.1: diameter
    ui->result_diameter->setText(v.toString()+" [um]");

    v.setValue((int unsigned long)(plot->fiber_taperlength));
    ui->result_length->setText(v.toString()+" [um]");

    v.setValue(0.01*(int unsigned long)(100.0*plot->fiber_max_angle));
    ui->result_angle->setText(v.toString()+" [mrad]");

    plot->segments=tthread->fiber_segments;
    plot->fiber_r0=tthread->fiber_r0;
   // plot->fiber_totlength = tthread->fiber_totlength;

    plot->update();
    tthread->updated--;
  }

  if (splashtimer>0)splashtimer--;



  if(tthread->isFinished())tthread->start();
  QTimer::singleShot(500, this, SLOT(anim()));
}

