#include "taperthread.h"


void TaperThread::initFiber()
{
  if(fiber_segments>N_max_) exit(1); //never happenes anyway...

  double dx=3.0*1.2*(0.5*(taper_x0+taper_L0))/((double)fiber_segments*fiber_segments*fiber_segments);
  for(int unsigned i=0; i<fiber_segments; i++)
  {
    fiber[i].r=fiber_r0;
    fiber[i].dl=(i-fiber_segments)*(i-fiber_segments)*dx;
    fiber[i].temp=ambienttemp;
  }
  hi_temp=low_temp=max_temp=ambienttemp;
  updated=2;
}

double TaperThread::L(double x)
{
  x*=2.0;

  double x3=x*x*x;
  double L=fabs(taper_L0+taper_L1*x+taper_L2*x*x+taper_L3*x3+taper_L4*x*x3+taper_L5*x*x*x3);

/*
// *****
  taper_L0= 5000.0;
  taper_L1= -0.5;

  double L;
  if(x<0.1*taper_x0)
    dummy=L=fabs(0.1*taper_x0-0.5*x);
  else
    L=fabs(0.7*taper_x0 - 0.5 *(x-0.1*taper_x0));
// *****
*/

  if(L<laser_w0)L=laser_w0; //make sure Lx never gets smaller than the waist!!!
  return(L);
}

void TaperThread::calcTemp(double x) //calculate fiber temp. Store it in fiber[].temp
{
  double Lx=L(x);
  double constants1=laser_power/(fiber_epsilon*sigma_*PI*2.0*laser_w0*Lx);
  double constants2=-0.5*PI*fiber_absorption;
  max_temp=ambienttemp;

  double xpos=0;
  for(int unsigned i=fiber_segments-1; i<65535; i--) //!!!!!!!!!!!
  {
    if(0.5*Lx>xpos)
    {
      fiber[i].temp=pow(constants1*(1.0-exp(constants2*fiber[i].r)),0.25);
      if(fiber[i].temp>max_temp)max_temp=fiber[i].temp;
    }
    else
    {
      fiber[i].temp=ambienttemp;
    }
    xpos+=fiber[i].dl; //add it here to make sure that at least the center segment is heated.
  }
  fiber_totlength=xpos;
}

void TaperThread::extrude()
{
  double visc_T0=-273.15-viscosity_T0;

  double dltot=0;
  for (int unsigned i =0; i<fiber_segments; i++)
  {
    double viscosity=(pow(10.0,viscosity_A+viscosity_B/(fiber[i].temp-visc_T0))); //calc viscosity based on temperature
    dltot+=(dlfiber[i].dl=(fiber[i].dl/(viscosity*fiber[i].r*fiber[i].r)));  //stretch per unit length and unit force
  }

  fiber_totlength=0;
  for(int unsigned i =0; i<fiber_segments; i++) //normalize the stretch
  {
    double dlnorm=dlfiber[i].dl/(dltot/dx);
    fiber[i].r=sqrt(fiber[i].r*fiber[i].r*fiber[i].dl/(fiber[i].dl+dlnorm)); //normalize the volume!
    fiber[i].dl+=dlnorm; //normalize the stretch!
    fiber_totlength+=fiber[i].dl;
  }
}

void TaperThread::doTheTapering()
{
  dx=0.5*taper_x0/taper_nofpulls;
  for(double x=0; x<=0.5*taper_x0; x+=dx)
  {
    if(current_job!=requested_job)return; //Abort! Values have changed.
    calcTemp(x); //updates the fiber temperature for all segments.
    extrude();  //extrude
    updated=2;
  }
  updated=2;
}


void TaperThread::run()
{
  while((requested_job_buffer=requested_job)!=current_job)
  {
    current_job=requested_job_buffer;
    initFiber();
    doTheTapering();

  }

  //Thread ends here. It will be restarted by 'MainWindow' if required.
}



