#ifndef __fiberTYPES__
#define __fiberTYPES__
typedef struct fiberelement
{
  double r;
  double dl;
  double temp;
}fiberelement;
#endif


#ifndef TAPERTHREAD_H
#define TAPERTHREAD_H

#include <math.h>
#include <QThread>

#define N_max_ 66000 //should be enough.

#define PI 3.141592653589793238462643383
#define sigma_ 5.67E-20 //[W/(um^2*K^4)] Stefan-Boltzman constant in funny units



class TaperThread : public QThread
{
public:

  void run();
  double L(double x); //make this public to allow plotting of the function.

// --- The following will be controlled by 'MainWindow' ---
  int requested_job; //requested_job is incremented by 'mainWindow' whenever any of the parameters are changed by the user.
  int current_job;

  int unsigned fiber_segments; //number of fiber segments
  double fiber_r0;         //[um] initial fiber radius
//  double fiber_sigma;      //
  double fiber_absorption; //[1/um] inverse absorption length of the fiber material @ laser wavelength
  double fiber_epsilon;    //[1] 'blackness' of glass (black body radiation)

  double taper_nofpulls;  //number of steps during the pulling
  double taper_x0;        //[um] total pull length
  double taper_L0;        //[um] initial heating length
  double taper_L1;        //[1]  L(x) = L0_ + L1*x + ...
  double taper_L2;        //[1]  L(x) = L0_ + L1*x + ...
  double taper_L3;        //[1]  L(x) = L0_ + L1*x + ...
  double taper_L4;        //[1]  L(x) = L0_ + L1*x + ...
  double taper_L5;
  double dummy;

  double viscosity_A;     //used for the calculation of the viscosity of the fiber as a function of temperature
  double viscosity_B;
  double viscosity_T0;

  double laser_power;     //[W] CO2 laser power
  double laser_w0;        //[um] laser beam radius at focus

  double ambienttemp;     //[K]

  char updated;  //set to 2 if fiber-array changed to indicate that plotting is requred.
  int cnt;
// ------------------------------------------------
  fiberelement fiber[N_max_];

  double max_temp,hi_temp,low_temp; //hi/low used for choosing the right color scheme. Maxtemp is shown as a result
  double fiber_totlength;  //just for plotting purposes
  //double fiber_taperlength; //this is just the length of the final taper
  //double fiber_mindiam;


private:
  void initFiber();
  void doTheTapering();
  void calcTemp(double x);
  void extrude();

    //if current_job != requested_job, fiber will be re-initialized and new job will be started
  int requested_job_buffer;
  double dx;

  fiberelement dlfiber[N_max_];


};

#endif // TAPERTHREAD_H
